
\newcommand{\vsucc}{\mathit{succ}}
\newcommand{\vtc}{\mathit{tc}}
\newcommand{\tc}[2]{#2^{(#1)}}
\newcommand{\Implies}{\Longrightarrow}
% \newcommand{\Exists}{\exists}

\begin{code}
-- # import ExamplePrelude
\end{code}

Natural numbers can be added to the framework by three new constants
$\mathit{Nat}, \mathit{zero}, \mathit{succ}$ plus an axiom for
mathematical induction.

\begin{code}
Nat  : Set 
zero : Nat
succ : Nat -> Nat
indNat (P : Nat -> Prop) : P zero -> ((n:Nat) -> P n -> P (succ n)) -> 
			     (m : Nat) -> P m
\end{code}

Now we fix a set $A$ and consider relations over $A$.  We want to
prove that the transitive closure of a symmetric relation is symmetric
as well.  We define the notion of symmetry and introduce a symbol for
relation composition.  We could define $R \circ R' = \lambda x \lambda
z \exists z.\, x\mathrel{R} y \land y \mathrel{R'} z$, but here we
only assume that a symmetric relation composed with itself is also
symmetric.

\begin{code}
A : Set
sym : (A->A->Prop) -> Prop
sym R = (x,y:A) -> R x y ==> R y x

(@)    : (A->A->Prop) -> (A->A->Prop) -> (A->A->Prop)
axSymO : (R:A->A->Prop) -> sym R -> sym (R @ R)
\end{code}

We define a monotone chain of approximations $\tc n R$ (in the source: $R\mathrel{\hat{}}n$) of the transitive closure, 
such that two elements will be related in the
transitive closure if they are related in some approximation.  The
main lemma states that all approximations are symmetric, if $R$ is
symmetric. 

\begin{code}
(^ )   :  (A->A->Prop) -> Nat -> (A->A->Prop)
axTc :  (R:A->A->Prop) -> (x,y:A) -> (n:Nat) -> 
            ((R ^ succ n) x y <=> (R ^ n) x y \/ ((R ^ n) @ (R ^ n)) x y)
         /\ ((R ^ zero) x y <=> R x y)

main : (R:A->A->Prop) -> sym R -> (n:Nat) -> sym (R ^ n)
main R h = indNat
            fol-plugin (h, axTc R)
            (\ n ih -> fol-plugin (h, axSymO (R ^ n) ih, axTc R, ih))
\end{code}
Induction is performed at the framework level, base and step case are
filled by Gandalf.  Pretty printed, Gandalf produces the following
proof of the step case:
%    14 o (R ^ n) (R ^ n) X Y  ==>  o (R ^ n) (R ^ n) Y X
%    17 tc (succ X) R Y Z  ==>  o (tc X R) (tc X R) Y Z \/ tc X R Y Z
%    18 o (tc X R) (tc X R) Y Z  ==>  tc (succ X) R Y Z
%    19 tc X R Y Z  ==>  tc (succ X) R Y Z
%    20 tc n R X Y  ==>  tc n R Y X
%    21 tc (succ n) R x0 y1
%    22 tc (succ n) R y1 x0  ==>  _|_
%    24 o (R ^ n) (R ^ n) x0 y1 \/ tc n R x0 y1 [hyper:17,21]
%    32 o (R ^ n) (R ^ n) y1 x0 \/ tc n R x0 y1 [hyper:14,24]
%    36 tc n R x0 y1 [hyper:18,32,cut:22]
%    38 tc n R y1 x0 [hyper:20,36]
%    39 _|_ [hyper:19,38,cut:22]
\[
\begin{array}{r@{\quad}rll}
  (1) & \forall x y. & (\tc n R \circ \tc n R)\,x\,y 
              \Implies (\tc n R \circ \tc n R)\,y\,x \\
  (2) & \forall m x y. & \tc {\vsucc\,m} R\,x\,y
              \Implies (\tc m R \circ \tc m R)\,x\,y 
                   \lor \tc m R\,x\,y  \\
  (3) & \forall m x y. & (\tc m R \circ \tc m R)\,x\,y 
                \Implies \tc {\vsucc\,m} R\,x\,y \\
  (4) & \forall m x y. & \tc m R\,x\,y 
                \Implies \tc {\vsucc\,m} R\,x\,y \\
  (5) & \forall x y. & \tc n R\,x\,y \Implies \tc n R\,y\,x \\
  (6) & & \tc {\vsucc\,n} R\,a\,b \\
  (7) & & \tc {\vsucc\,n} R \,b\,a \Implies {\perp} \\
  (8) & & (\tc n R \circ \tc n R)\,a\,b \lor \tc n R\,a\,b & (2),(6)
  \\
  (9) & & (\tc n R \circ \tc n R)\,b\,a \lor \tc n R\,a\,b & (1),(8)
  \\
 (10) & & \tc n R\,a\,b & (3),(7),(9) \\
 (11) & & \tc n R\,b\,a & (5),(10) \\
 (12) & & \perp & (4),(7),(11) \\
\end{array}
\]
%\[
%\begin{array}{r@{\quad}rll}
%  (1) & \forall X Y. & (\tc n R \circ \tc n R)\,X\,Y 
%              \Implies (\tc n R \circ \tc n R)\,Y\,X \\
%  (2) & \forall N X Y. & \tc {\vsucc\,n} R\,X\,Y
%              \Implies (\tc n R \circ \tc n R)\,X\,Y 
%                   \lor \tc n R\,X\,Y  \\
%  (3) & \forall N X Y. & (\tc n R \circ \tc n R)\,X\,Y 
%                \Implies \tc {\vsucc\,n} R\,X\,Y \\
%  (4) & \forall N X Y. & \tc n R\,X\,Y 
%                \Implies \tc {\vsucc\,n} R\,X\,Y \\
%  (5) & \forall N X Y. & \tc n R\,X\,Y \Implies \tc n R\,Y\,X \\
%  (6) & & \tc {\vsucc\,n}\,a\,b \\
%  (7) & & \tc {\vsucc\,n}\,b\,a \Implies {\perp} \\
%  (8) & & (\tc n R \circ \tc n R)\,a\,b \lor \tc n R\,a\,b & (2),(6)
%  \\
%  (9) & & (\tc n R \circ \tc n R)\,b\,a \lor \tc n R\,a\,b & (1),(8)
%  \\
% (10) & & \tc n R\,a\,b & (3),(7),(9) \\
% (11) & & \tc n R\,b\,a & (5),(10) \\
% (12) & & \perp & (4),(7),(11) \\
%\end{array}
%\]
\newcommand{\TC}{\mathit{TC}}

The transitive closure is now defined as $\TC\,R\,x\,y = \exists n.\,
\tc n R x y$.  To formalize this, we add existential quantification and
its proof rules.
The final theorem demostrates how existential quantification can be
handled in the framework.

\begin{code}
Exists (X:Set) : (X -> Prop) -> Prop
existsI (X:Set)(P:X->Prop) : (x:X) -> P x -> Exists P
existsE (X:Set)(P:X->Prop)(C:Prop) :
    Exists P -> ((x:X) -> P x -> C) -> C

TC : (A->A->Prop) -> A->A->Prop
TC R x y = Exists (\ n -> (R ^ n) x y)

thm : (R:A->A->Prop) -> sym R -> sym (TC R)
thm R h x y = impI (\ p ->
	      existsE p (\ n q -> existsI n fol-plugin(q, main R h n)))
\end{code}


%axTcZero :  (R:A->A->Prop) -> (x,y:A) -> (tc zero R) x y <=> R x y
%axTcSucc :  (R:A->A->Prop) -> (n:Nat) -> (x,y:A) -> 
%             (tc (succ n) R) x y <=> ((R ^ n) @ (R ^ n)) x y
