
module Plugins.FOL.Errors where

import Text.PrettyPrint

import Syntax.Internal	( Term, Name )
import Syntax.Values	( Value, Type, SortValue )
import Syntax.Pretty	( pretty )
import Syntax.Position	( noRange, getRange )

import Utils.Error
import Plugins.FOL.Types
import Plugins.FOL.Constants

-- Whats and whens --------------------------------------------------------

whenTranslatingToFOL :: Doc
whenTranslatingToFOL =
    fsep $ pwords "when translating a goal" ++ -- [pretty v] ++
	   pwords "to first order logic"

higherOrderQuantification :: Name -> Type -> Doc
higherOrderQuantification x t =
    fsep $ pwords "quantification over" ++ [pretty x, text "in"] ++ [pretty t] ++
	   pwords "is not first order"

-- Errors -----------------------------------------------------------------

notAFOLPredicateError :: Value -> ErrorFun
notAFOLPredicateError v e =
    e	{ errWhat   = fsep $ pwords "the type" ++ [pretty v] ++
			     pwords "is not a valid FOL predicate"
	, errWhy    = fsep $ pwords "it is not of the form (x::A) -> .. -> Holds p"
	}

notAFOLPropositionError :: Value -> ErrorFun
notAFOLPropositionError v e =
    e	{ errWhat   = fsep $ pwords "the proposition" ++ [pretty v] ++
			     pwords "is not first order"
	}

notAFOLTermError :: Value -> ErrorFun
notAFOLTermError v e =
    e	{ errWhat   = fsep $ pwords "the value" ++ [pretty v] ++
			     pwords "is not a first order term"
	}

noInferenceForFOLError :: ErrorFun
noInferenceForFOLError e =
    e	{ errWhat   = fsep $ pwords "type inference failed"
	, errWhy    = fsep $ pwords "the type of a first order goal cannot be inferred"
	}

failedToProveFirstOrderGoalsError :: [(FOLGoal, Maybe CounterModel)] -> ErrorFun
failedToProveFirstOrderGoalsError gs e =
    e	{ errWhere  = noRange
	, errWhen   = fsep $ pwords "solving first order goals"
	, errWhat   = vcat $ fsep (pwords "the prover failed to prove") : map display gs
	}
    where
	display (g,Nothing) = nest 2 $ pretty $ getRange g
	display (g, Just m) = nest 2 $
				vcat
				[ pretty $ getRange g
				, nest 2 $ text "Counter model:"
				, nest 4 $ vcat $ map text m
				]

notOfTypePropError :: Type -> SortValue -> ErrorFun
notOfTypePropError v t e =
    e	{ errWhat   = fsep $ pwords "the type" ++ [pretty v] ++
			     pwords "is not a first order formula"
	, errWhy    = fsep $ pwords "it has sort" ++ [pretty t] ++
			     pwords "when it should have sort" ++ pwords fol_Prop
	}

