{-# OPTIONS -fglasgow-exts -fallow-overlapping-instances #-}
module TypeChecker.MetaVars.Free where

import Syntax.Values
import TypeChecker.Monad
import TypeChecker.Values.Refresh
import Utils.Monads

class FreeMetas a where
    freeMetas :: a -> TC [MetaRef]

instance FreeMetas Value where
    freeMetas v =
	do  v' <- refreshTop v
	    case vView v' of
		VId _	    -> return []
		VBId _	    -> return []
		VTop	    -> return []
		VDef _ _    -> return []
		VCon _ _ _  -> return []
		VSort _	    -> return []
		VMeta m	    -> return [m]
		VApp v w    -> freeMetas (v,w)
		VPi v f	    -> freeMetas (v,f)
		VLam f	    -> freeMetas f
		VPair v w   -> freeMetas (v,w)
		VProj v _   -> freeMetas v
		VSigma v f  -> freeMetas (v,f)
		VStruct xs  -> freeMetas xs

instance FreeMetas Type where
    freeMetas = freeMetas . unEl

instance FreeMetas a => FreeMetas (Abs a) where
    freeMetas f = freeMetas (absBody f)

instance FreeMetas ClauseValue where
    freeMetas (ClauseVal _ v) = freeMetas v

instance FreeMetas DefValue where
    freeMetas (DefVal _ _ _ _ cs t) = freeMetas (cs,t)

instance FreeMetas Int where
    freeMetas _ = return []

instance FreeMetas String where
    freeMetas _ = return []

instance (FreeMetas a, FreeMetas b) => FreeMetas (a,b) where
    freeMetas (x,y) = (++) <$> freeMetas x <*> freeMetas y

instance (FreeMetas a, FreeMetas b, FreeMetas c) => FreeMetas (a,b,c) where
    freeMetas (x,y,z) = freeMetas (x,(y,z))

instance FreeMetas a => FreeMetas [a] where
    freeMetas xs = concat <$> mapM freeMetas xs

isClosed :: FreeMetas a => a -> TC Bool
isClosed x = null <$> freeMetas x


