{-# OPTIONS -fglasgow-exts -fallow-overlapping-instances #-}

module TypeChecker.Values.Refresh where

import Control.Monad.Trans
import Data.IORef

import Syntax.Internal
import Syntax.Values
import TypeChecker.Context
import TypeChecker.Monad
import TypeChecker.Abstractions
import Utils.Monads

-- | Only refresh top-level metas
refreshTop :: Value -> TC Value
refreshTop v =
    case vAppProjView v of
	VAPVMeta m vs	->
	    do  mv <- liftIO $ readIORef m
		case mv of
		    Open	    -> return v
		    Instantiated v  -> refreshTop (v `vAppOrProjs` vs)
	_ -> return v

refreshSpine :: Value -> TC Value
refreshSpine v =
    do	v' <- refreshTop v
	case appView v' of
	    v:vs    -> vApps v <$> mapM refreshTop vs
	    _	    -> error "impossible: TypeChecker.Values.Refresh.refreshSpine"
    where
	appView v =
	    case vView v of
		VApp v w    -> appView v ++ [w]
		_	    -> [v]

class Refresh a where
    refresh :: a -> TC a

instance Refresh AppOrProj where
    refresh (IsApp v)  = IsApp <$> refresh v
    refresh (IsProj p) = return $ IsProj p

instance Refresh Value where
    refresh v =
	case vAppProjView v of
	    VAPVId x vs   -> vAppOrProjs (vId x) <$> refresh vs
	    VAPVBId n vs  -> vAppOrProjs (vBId n) <$> refresh vs
	    VAPVMeta m vs ->
		do  mv <- liftIO $ readIORef m
		    case mv of
			Open	       -> return v
			Instantiated v -> refresh (vAppOrProjs v vs)
			    -- Note: it's important not to refresh the arguments to
			    -- instantiated meta variables (might lead to non-termination)

	    VAPVDef x h vs    -> vAppOrProjs (vDef x h) <$> refresh vs
	    VAPVCon x h ar vs -> vAppOrProjs (vCon x h ar) <$> refresh vs
	    VAPVPi v cl	      -> vPi <$> refresh v <*> refresh cl
	    VAPVLam cl	      -> vLam <$> refresh cl
	    VAPVSort s	      -> return v
	    VAPVTop	      -> return vTop
	    VAPVPair v w      -> vPair <$> refresh v <*> refresh w
	    VAPVSigma w f     -> vSigma <$> refresh w <*> refresh f
	    VAPVStruct xs     -> vStruct <$> refresh xs

instance Refresh Type where
    refresh t = typeMapM refresh t

instance Refresh ContextEntry where
    refresh (Defined v t)   = Defined <$> refresh v <*> refresh t
    refresh (Abstract t)    = Abstract <$> refresh t

instance Refresh a => Refresh (Abs a) where
    refresh = absMapM refresh

instance Refresh ClauseValue where
    refresh (ClauseVal ps v) = ClauseVal ps <$> refresh v

instance Refresh DefValue where
    refresh (DefVal h ar fv i cs t) = DefVal h ar fv i <$> refresh cs <*> refresh t

instance Refresh Name where
    refresh x = return x

instance Refresh String where
    refresh x = return x

instance Refresh Int where
    refresh n = return n

instance Refresh a => Refresh [a] where
    refresh xs = mapM refresh xs

instance (Refresh a, Refresh b) => Refresh (a,b) where
    refresh (x,y) = (,) <$> refresh x <*> refresh y


